###########################################################################
#
#  Copyright 2020 Google LLC
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
###########################################################################

PartnerCosts_Schema = [
    {
        'name': 'cpm_fee_1_advertiser_micros',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_2_advertiser_micros',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_3_advertiser_micros',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_4_advertiser_micros',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_5_advertiser_micros',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_1_millis',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_2_millis',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_3_millis',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_4_millis',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_5_millis',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_1_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_2_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_3_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_4_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_5_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_1_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_2_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_3_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_4_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_5_cost_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_1_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_2_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_3_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_4_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'cpm_fee_5_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_1_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_2_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_3_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_4_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
    {
        'name': 'media_fee_percent_5_bill_to_type',
        'type': 'INTEGER',
        'mode': 'NULLABLE',
    },
]
