#!/usr/bin/env python

###########################################################################
#
#  Copyright 2020 Google LLC
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
###########################################################################

import os
import sys

if __name__ == '__main__':
  os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'ui.settings')
  try:
    from django.core.management import execute_from_command_line
  except ImportError:
    # The above import may fail for some other reason. Ensure that the
    # issue is really that Django is missing to avoid masking other
    # exceptions on Python 2.
    try:
      import django
    except ImportError:
      raise ImportError(
          "Couldn't import Django. Are you sure it's installed and "
          'available on your PYTHONPATH environment variable? Did you '
          'forget to activate a virtual environment?')
    raise
  execute_from_command_line(sys.argv)
