###########################################################################
#
#  Copyright 2020 Google LLC
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
###########################################################################

Floodlight_Schema = [{
    'name': 'Partitiontime',
    'type': 'TIMESTAMP',
    'mode': 'NULLABLE'
}, {
    'name': 'Event_Time',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'User_Id',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Advertiser_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Campaign_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Ad_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Creative_Version',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Site_Id_Dcm',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Browser_Platform_Version',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Event_Type',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Event_Sub_Type',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Auction_Id',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Request_Time',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Advertiser_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Line_Item_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Creative_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Url',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Site_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Language',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Adx_Page_Categories',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Matching_Targeted_Keywords',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Exchange_Id',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Attributed_Inventory_Source_External_Id',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Attributed_Inventory_Source_Is_Public',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Ad_Position',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Country_Code',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Browser_Platform_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Net_Speed',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Matching_Targeted_Segments',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Isp_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Mobile_Model_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Designated_Market_Area_Dma_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_City_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Browser_Timezone_Offset_Minutes',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dbm_Mobile_Make_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Activity_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Creative_Pixel_Size',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Total_Conversions',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}]
