# Generated by Django 2.2.13 on 2020-11-20 07:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('project', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='key',
            field=models.CharField(default='', max_length=50),
        ),
    ]
