###########################################################################
#
#  Copyright 2020 Google LLC
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
###########################################################################

Activity_Metrics_Schema = [{
    'name': 'Click_Through_Conversions',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Click_Through_Revenue',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'View_Through_Conversions',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'View_Through_Revenue',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Click_Through_Conversions_Cross_Environment',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Click_Through_Revenue_Cross_Environment',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Click_Through_Conversion_Events_Cross_Environment',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Total_Conversions_Cross_Environment',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Total_Revenue_Cross_Environment',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Total_Conversion_Events_Cross_Environment',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'View_Through_Conversions_Cross_Environment',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'View_Through_Revenue_Cross_Environment',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'View_Through_Conversion_Events_Cross_Environment',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dynamic_Element_Click_Through_Conversions',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dynamic_Element_Total_Conversions',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Dynamic_Element_View_Through_Conversions',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Natural_Search_Actions',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Natural_Search_Revenue',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Natural_Search_Transactions',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Paid_Search_Action_Conversion_Percentage',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Paid_Search_Actions',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Paid_Search_Average_Cost_Per_Action',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Paid_Search_Average_Cost_Per_Transaction',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Paid_Search_Average_Dcm_Transaction_Amount',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Paid_Search_Revenue',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Paid_Search_Spend_Per_Transaction_Revenue',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Paid_Search_Transaction_Conversion_Percentage',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Paid_Search_Transaction_Revenue_Per_Spend',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Paid_Search_Transactions',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Total_Conversions',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}, {
    'name': 'Total_Revenue',
    'type': 'FLOAT',
    'mode': 'NULLABLE'
}]
