###########################################################################
#
#  Copyright 2020 Google LLC
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
###########################################################################

DCM_Field_Lookup = {
    'Video_Unmutes':
        'INTEGER',
    'Zip_Postal_Code':
        'STRING',
    'Path_Length':
        'INTEGER',
    'Measurable_Impressions_For_Audio':
        'INTEGER',
    'Average_Interaction_Time':
        'FLOAT',
    'Invalid_Impressions':
        'INTEGER',
    'Floodlight_Variable_40':
        'STRING',
    'Floodlight_Variable_41':
        'STRING',
    'Floodlight_Variable_42':
        'STRING',
    'Floodlight_Variable_43':
        'STRING',
    'Floodlight_Variable_44':
        'STRING',
    'Floodlight_Variable_45':
        'STRING',
    'Floodlight_Variable_46':
        'STRING',
    'Floodlight_Variable_47':
        'STRING',
    'Floodlight_Variable_48':
        'STRING',
    'Floodlight_Variable_49':
        'STRING',
    'Clicks':
        'INTEGER',
    'Warnings':
        'INTEGER',
    'Paid_Search_Advertiser_Id':
        'INTEGER',
    'Video_Third_Quartile_Completions':
        'INTEGER',
    'Video_Progress_Events':
        'INTEGER',
    'Cost_Per_Revenue':
        'FLOAT',
    'Total_Interaction_Time':
        'INTEGER',
    'Roadblock_Impressions':
        'INTEGER',
    'Video_Replays':
        'INTEGER',
    'Keyword':
        'STRING',
    'Full_Screen_Video_Plays':
        'INTEGER',
    'Active_View_Impression_Distribution_Not_Measurable':
        'FLOAT',
    'Unique_Reach_Total_Reach':
        'INTEGER',
    'Has_Exits':
        'BOOLEAN',
    'Paid_Search_Engine_Account':
        'STRING',
    'Operating_System_Version':
        'STRING',
    'Campaign':
        'STRING',
    'Active_View_Percent_Visible_At_Start':
        'FLOAT',
    'Active_View_Not_Viewable_Impressions':
        'INTEGER',
    'Twitter_Offers_Accepted':
        'INTEGER',
    'Interaction_Type':
        'STRING',
    'Activity_Delivery_Status':
        'FLOAT',
    'Video_Companion_Clicks':
        'INTEGER',
    'Floodlight_Paid_Search_Average_Cost_Per_Transaction':
        'FLOAT',
    'Paid_Search_Agency_Id':
        'INTEGER',
    'Asset':
        'STRING',
    'Active_View_Audible_Amp_Fully_On_Screen_For_Half_Of_Duration_15_Sec_Cap_Impressions':
        'INTEGER',
    'User_List_Current_Size':
        'STRING',
    'Exit_Url':
        'STRING',
    'Natural_Search_Revenue':
        'FLOAT',
    'Retweets':
        'INTEGER',
    'Full_Screen_Impressions':
        'INTEGER',
    'Audio_Unmutes':
        'INTEGER',
    'Dynamic_Element_4_Field_Value_2':
        'STRING',
    'Dynamic_Element_4_Field_Value_3':
        'STRING',
    'Dynamic_Element_4_Field_Value_1':
        'STRING',
    'Creative_Start_Date':
        'DATE',
    'Small_Video_Player_Size_Impressions':
        'INTEGER',
    'Audio_Replays':
        'INTEGER',
    'Video_Player_Size_Avg_Width':
        'INTEGER',
    'Rich_Media_Standard_Event_Count':
        'INTEGER',
    'Publisher_Problems':
        'INTEGER',
    'Paid_Search_Advertiser':
        'STRING',
    'Has_Video_Completions':
        'BOOLEAN',
    'Cookie_Reach_Duplicate_Impression_Reach':
        'INTEGER',
    'Audio_Third_Quartile_Completions':
        'INTEGER',
    'Package_Roadblock_Strategy':
        'STRING',
    'Video_Midpoints':
        'INTEGER',
    'Click_Through_Conversion_Events_Cross_Environment':
        'INTEGER',
    'Video_Pauses':
        'INTEGER',
    'Trueview_Views':
        'INTEGER',
    'Interaction_Count_Mobile_Static_Image':
        'INTEGER',
    'Dynamic_Element_Click_Rate':
        'FLOAT',
    'Dynamic_Element_1_Field_6_Value':
        'STRING',
    'Dynamic_Element_4_Value':
        'STRING',
    'Creative_End_Date':
        'DATE',
    'Dynamic_Element_4_Field_3_Value':
        'STRING',
    'Dynamic_Field_Value_3':
        'STRING',
    'Mobile_Carrier':
        'STRING',
    'U_Value':
        'STRING',
    'Average_Display_Time':
        'FLOAT',
    'Custom_Variable':
        'STRING',
    'Video_Interactions':
        'INTEGER',
    'Average_Expansion_Time':
        'FLOAT',
    'Email_Shares':
        'INTEGER',
    'Flight_Booked_Rate':
        'STRING',
    'Impression_Count':
        'INTEGER',
    'Site_Id_Dcm':
        'INTEGER',
    'Dynamic_Element_3_Value_Id':
        'STRING',
    'Paid_Search_Legacy_Keyword_Id':
        'INTEGER',
    'View_Through_Conversion_Events_Cross_Environment':
        'INTEGER',
    'Counters':
        'INTEGER',
    'Floodlight_Paid_Search_Average_Cost_Per_Action':
        'FLOAT',
    'Activity_Group_Id':
        'INTEGER',
    'Active_View_Percent_Audible_And_Visible_At_Midpoint':
        'FLOAT',
    'Click_Count':
        'INTEGER',
    'Video_4A_39_S_Ad_Id':
        'STRING',
    'Total_Interactions':
        'INTEGER',
    'Active_View_Viewable_Impressions':
        'INTEGER',
    'Natural_Search_Engine_Property':
        'STRING',
    'Video_Views':
        'INTEGER',
    'Domain':
        'STRING',
    'Total_Conversion_Events_Cross_Environment':
        'INTEGER',
    'Dbm_Advertiser':
        'STRING',
    'Companion_Impressions':
        'INTEGER',
    'View_Through_Conversions_Cross_Environment':
        'INTEGER',
    'Dynamic_Element_5_Field_Value_3':
        'STRING',
    'Dynamic_Element_5_Field_Value_2':
        'STRING',
    'Dynamic_Element_5_Field_Value_1':
        'STRING',
    'Dynamic_Field_Value_1':
        'STRING',
    'Active_View_Impression_Distribution_Viewable':
        'FLOAT',
    'Creative_Field_2':
        'STRING',
    'Twitter_Leads_Generated':
        'INTEGER',
    'Paid_Search_External_Campaign_Id':
        'INTEGER',
    'Creative_Field_8':
        'STRING',
    'Interaction_Count_Paid_Search':
        'INTEGER',
    'Activity_Group':
        'STRING',
    'Video_Player_Location_Avg_Pixels_From_Top':
        'INTEGER',
    'Interaction_Number':
        'INTEGER',
    'Dynamic_Element_3_Value':
        'STRING',
    'Cookie_Reach_Total_Reach':
        'INTEGER',
    'Cookie_Reach_Exclusive_Click_Reach':
        'INTEGER',
    'Creative_Field_5':
        'STRING',
    'Cookie_Reach_Overlap_Impression_Reach':
        'INTEGER',
    'Paid_Search_Ad_Group':
        'STRING',
    'Interaction_Count_Rich_Media':
        'INTEGER',
    'Dynamic_Element_Total_Conversions':
        'INTEGER',
    'Transaction_Count':
        'INTEGER',
    'Num_Value':
        'STRING',
    'Cookie_Reach_Overlap_Total_Reach':
        'INTEGER',
    'Floodlight_Attribution_Type':
        'STRING',
    'Cookie_Reach_Exclusive_Impression_Reach_Percent':
        'FLOAT',
    'Html5_Impressions':
        'INTEGER',
    'Served_Pixel_Density':
        'STRING',
    'Has_Full_Screen_Video_Plays':
        'BOOLEAN',
    'Interaction_Count_Static_Image':
        'INTEGER',
    'Has_Video_Companion_Clicks':
        'BOOLEAN',
    'Twitter_Video_100Percent_In_View_For_3_Seconds':
        'INTEGER',
    'Paid_Search_Ad':
        'STRING',
    'Paid_Search_Visits':
        'INTEGER',
    'Audio_Pauses':
        'INTEGER',
    'Creative_Pixel_Size':
        'STRING',
    'Flight_Start_Date':
        'DATE',
    'Active_View_Percent_Of_Third_Quartile_Impressions_Visible':
        'FLOAT',
    'Natural_Search_Transactions':
        'FLOAT',
    'Cookie_Reach_Impression_Reach':
        'INTEGER',
    'Dynamic_Field_Value_4':
        'STRING',
    'Twitter_Line_Item_Id':
        'INTEGER',
    'Has_Video_Stops':
        'BOOLEAN',
    'Active_View_Percent_Audible_And_Visible_At_Completion':
        'FLOAT',
    'Paid_Search_Labels':
        'STRING',
    'Twitter_Creative_Type':
        'STRING',
    'Operating_System':
        'STRING',
    'Dynamic_Element_3_Field_4_Value':
        'STRING',
    'Hours_Since_First_Interaction':
        'INTEGER',
    'Asset_Category':
        'STRING',
    'Active_View_Measurable_Impressions':
        'INTEGER',
    'Package_Roadblock':
        'STRING',
    'Large_Video_Player_Size_Impressions':
        'INTEGER',
    'Paid_Search_Actions':
        'FLOAT',
    'Active_View_Percent_Visible_At_Midpoint':
        'FLOAT',
    'Has_Full_Screen_Views':
        'BOOLEAN',
    'Backup_Image':
        'INTEGER',
    'Likes':
        'INTEGER',
    'Dbm_Line_Item':
        'STRING',
    'Active_View_Percent_Audible_Impressions':
        'FLOAT',
    'Flight_Booked_Cost':
        'STRING',
    'Other_Twitter_Engagements':
        'INTEGER',
    'Audio_Completions':
        'INTEGER',
    'Click_Rate':
        'FLOAT',
    'Cost_Per_Activity':
        'FLOAT',
    'Dynamic_Element_2_Value':
        'STRING',
    'Cookie_Reach_Exclusive_Impression_Reach':
        'INTEGER',
    'Content_Category':
        'STRING',
    'Twitter_Video_50Percent_In_View_For_2_Seconds':
        'INTEGER',
    'Total_Revenue_Cross_Environment':
        'FLOAT',
    'Unique_Reach_Click_Reach':
        'INTEGER',
    'Has_Video_Replays':
        'BOOLEAN',
    'Twitter_Creative_Id':
        'INTEGER',
    'Has_Counters':
        'BOOLEAN',
    'Dynamic_Element_4_Value_Id':
        'STRING',
    'Dynamic_Element_4_Field_1_Value':
        'STRING',
    'Has_Video_Interactions':
        'BOOLEAN',
    'Video_Player_Size':
        'STRING',
    'Active_View_Audible_Amp_Fully_On_Screen_For_Half_Of_Duration_Trueview_Impressions':
        'INTEGER',
    'Advertiser_Id':
        'INTEGER',
    'Rich_Media_Clicks':
        'INTEGER',
    'Floodlight_Variable_59':
        'STRING',
    'Floodlight_Variable_58':
        'STRING',
    'Floodlight_Variable_57':
        'STRING',
    'Floodlight_Variable_56':
        'STRING',
    'Floodlight_Variable_55':
        'STRING',
    'Floodlight_Variable_54':
        'STRING',
    'Floodlight_Variable_53':
        'STRING',
    'Floodlight_Variable_52':
        'STRING',
    'Floodlight_Variable_51':
        'STRING',
    'Floodlight_Variable_50':
        'STRING',
    'Cookie_Reach_Incremental_Total_Reach':
        'INTEGER',
    'Playback_Method':
        'STRING',
    'Has_Interactive_Impressions':
        'BOOLEAN',
    'Paid_Search_Average_Position':
        'FLOAT',
    'Floodlight_Variable_96':
        'STRING',
    'Package_Roadblock_Id':
        'INTEGER',
    'Recalculated_Attribution_Type':
        'STRING',
    'Dbm_Advertiser_Id':
        'INTEGER',
    'Floodlight_Variable_100':
        'STRING',
    'Cookie_Reach_Exclusive_Total_Reach_Percent':
        'FLOAT',
    'Active_View_Impression_Distribution_Not_Viewable':
        'FLOAT',
    'Floodlight_Variable_3':
        'STRING',
    'Floodlight_Variable_2':
        'STRING',
    'Floodlight_Variable_1':
        'STRING',
    'Floodlight_Variable_7':
        'STRING',
    'Floodlight_Variable_6':
        'STRING',
    'Floodlight_Variable_5':
        'STRING',
    'Floodlight_Variable_4':
        'STRING',
    'Rendering_Id':
        'INTEGER',
    'Floodlight_Variable_9':
        'STRING',
    'Floodlight_Variable_8':
        'STRING',
    'Cookie_Reach_Incremental_Impression_Reach':
        'INTEGER',
    'Active_View_Percent_Visible_At_Third_Quartile':
        'FLOAT',
    'Reporting_Problems':
        'INTEGER',
    'Package_Roadblock_Total_Booked_Units':
        'STRING',
    'Active_View_Percent_Visible_At_Completion':
        'FLOAT',
    'Has_Video_Midpoints':
        'BOOLEAN',
    'Dynamic_Element_4_Field_4_Value':
        'STRING',
    'Percentage_Of_Measurable_Impressions_For_Video_Player_Location':
        'FLOAT',
    'Campaign_End_Date':
        'DATE',
    'Placement_External_Id':
        'STRING',
    'Cost_Per_Click':
        'FLOAT',
    'Cookie_Reach_Overlap_Click_Reach_Percent':
        'FLOAT',
    'Active_View_Percent_Full_Screen':
        'FLOAT',
    'Hour':
        'STRING',
    'Click_Through_Revenue':
        'FLOAT',
    'Video_Skips':
        'INTEGER',
    'Paid_Search_Click_Rate':
        'FLOAT',
    'Has_Video_Views':
        'BOOLEAN',
    'Dbm_Cost_Account_Currency':
        'FLOAT',
    'Flight_End_Date':
        'DATE',
    'Has_Video_Plays':
        'BOOLEAN',
    'Paid_Search_Clicks':
        'INTEGER',
    'Creative_Field_9':
        'STRING',
    'Manual_Closes':
        'INTEGER',
    'Creative_Field_7':
        'STRING',
    'Creative_Field_6':
        'STRING',
    'App':
        'STRING',
    'Creative_Field_4':
        'STRING',
    'Creative_Field_3':
        'STRING',
    'Campaign_Start_Date':
        'DATE',
    'Creative_Field_1':
        'STRING',
    'Content_Classifier':
        'STRING',
    'Cookie_Reach_Duplicate_Click_Reach':
        'INTEGER',
    'Site_Dcm':
        'STRING',
    'Digital_Content_Label':
        'STRING',
    'Has_Manual_Closes':
        'BOOLEAN',
    'Has_Timers':
        'BOOLEAN',
    'Impressions':
        'INTEGER',
    'Classified_Impressions':
        'INTEGER',
    'Dbm_Site_Id':
        'INTEGER',
    'Dynamic_Element_2_Field_1_Value':
        'STRING',
    'Floodlight_Variable_72':
        'STRING',
    'Creative':
        'STRING',
    'Asset_Orientation':
        'STRING',
    'Custom_Variable_Count_1':
        'INTEGER',
    'Video_Stops':
        'INTEGER',
    'Paid_Search_Ad_Id':
        'INTEGER',
    'Cookie_Reach_Overlap_Total_Reach_Percent':
        'FLOAT',
    'Click_Delivery_Status':
        'FLOAT',
    'Dynamic_Element_Impressions':
        'INTEGER',
    'Interaction_Count_Click_Tracker':
        'INTEGER',
    'Placement_Total_Booked_Units':
        'STRING',
    'Date':
        'DATE',
    'Twitter_Placement_Type':
        'STRING',
    'Total_Revenue':
        'FLOAT',
    'Recalculated_Attributed_Interaction':
        'STRING',
    'Ad_Type':
        'STRING',
    'Social_Engagement_Rate':
        'FLOAT',
    'Dynamic_Element_5_Field_1_Value':
        'STRING',
    'Dynamic_Profile_Id':
        'INTEGER',
    'Active_View_Impressions_Visible_10_Seconds':
        'INTEGER',
    'Interaction_Count_Video':
        'INTEGER',
    'Dynamic_Element_5_Value':
        'STRING',
    'Video_Player_Size_Avg_Height':
        'INTEGER',
    'Creative_Type':
        'STRING',
    'Campaign_External_Id':
        'STRING',
    'Dynamic_Element_Click_Through_Conversions':
        'INTEGER',
    'Conversion_Url':
        'STRING',
    'Floodlight_Variable_89':
        'STRING',
    'Floodlight_Variable_84':
        'STRING',
    'Floodlight_Variable_85':
        'STRING',
    'Floodlight_Variable_86':
        'STRING',
    'Floodlight_Variable_87':
        'STRING',
    'Floodlight_Variable_80':
        'STRING',
    'Floodlight_Variable_81':
        'STRING',
    'Floodlight_Variable_82':
        'STRING',
    'Floodlight_Variable_83':
        'STRING',
    'Active_View_Audible_Amp_Fully_On_Screen_For_Half_Of_Duration_Trueview_Measurable_Impressions':
        'INTEGER',
    'Floodlight_Variable_66':
        'STRING',
    'Floodlight_Variable_67':
        'STRING',
    'Floodlight_Variable_64':
        'STRING',
    'Floodlight_Variable_65':
        'STRING',
    'Floodlight_Variable_62':
        'STRING',
    'Floodlight_Variable_63':
        'STRING',
    'Floodlight_Variable_60':
        'STRING',
    'Active_View_Eligible_Impressions':
        'INTEGER',
    'Dynamic_Element_3_Field_Value_1':
        'STRING',
    'Dynamic_Element_3_Field_Value_3':
        'STRING',
    'Dynamic_Element_3_Field_Value_2':
        'STRING',
    'Floodlight_Variable_68':
        'STRING',
    'Floodlight_Variable_69':
        'STRING',
    'Floodlight_Variable_13':
        'STRING',
    'Floodlight_Variable_12':
        'STRING',
    'Floodlight_Variable_11':
        'STRING',
    'Floodlight_Variable_10':
        'STRING',
    'Floodlight_Variable_17':
        'STRING',
    'Floodlight_Variable_16':
        'STRING',
    'Floodlight_Variable_15':
        'STRING',
    'Floodlight_Variable_14':
        'STRING',
    'Floodlight_Variable_19':
        'STRING',
    'Floodlight_Variable_18':
        'STRING',
    'Audience_Targeted':
        'STRING',
    'Total_Conversions_Cross_Environment':
        'INTEGER',
    'Interaction_Count_Mobile_Rich_Media':
        'INTEGER',
    'Active_View_Percent_Viewable_Impressions':
        'FLOAT',
    'Dynamic_Element_1_Field_1_Value':
        'STRING',
    'Active_View_Audible_Amp_Fully_On_Screen_For_Half_Of_Duration_Trueview_Rate':
        'FLOAT',
    'Has_Video_Skips':
        'BOOLEAN',
    'Dynamic_Element_5_Field_2_Value':
        'STRING',
    'Twitter_Buy_Now_Clicks':
        'INTEGER',
    'Creative_Groups_2':
        'STRING',
    'Creative_Groups_1':
        'STRING',
    'Campaign_Id':
        'INTEGER',
    'Twitter_Campaign_Id':
        'INTEGER',
    'Active_View_Audible_Amp_Fully_On_Screen_For_Half_Of_Duration_15_Sec_Cap_Rate':
        'FLOAT',
    'Dynamic_Element_1_Field_5_Value':
        'STRING',
    'Paid_Search_Match_Type':
        'STRING',
    'Activity_Per_Thousand_Impressions':
        'FLOAT',
    'Has_Expansions':
        'BOOLEAN',
    'Dbm_Creative_Id':
        'INTEGER',
    'Active_View_Percent_Audible_And_Visible_At_Start':
        'FLOAT',
    'Booked_Viewable_Impressions':
        'FLOAT',
    'Dynamic_Element_1_Field_2_Value':
        'STRING',
    'Paid_Search_Cost':
        'FLOAT',
    'Dynamic_Element_5_Field_5_Value':
        'STRING',
    'Floodlight_Paid_Search_Spend_Per_Transaction_Revenue':
        'FLOAT',
    'Active_View_Percent_Of_Midpoint_Impressions_Audible_And_Visible':
        'FLOAT',
    'Dynamic_Element_4':
        'STRING',
    'Dynamic_Element_5':
        'STRING',
    'Dynamic_Element_2':
        'STRING',
    'Click_Through_Conversions':
        'FLOAT',
    'Dynamic_Element_1':
        'STRING',
    'Dynamic_Element_4_Field_6_Value':
        'STRING',
    'Attributed_Event_Platform_Type':
        'STRING',
    'Audio_Midpoints':
        'INTEGER',
    'Attributed_Event_Connection_Type':
        'STRING',
    'Dynamic_Element_1_Value':
        'STRING',
    'Measurable_Impressions_For_Video_Player_Location':
        'INTEGER',
    'Audio_Companion_Impressions':
        'INTEGER',
    'Video_Full_Screen':
        'INTEGER',
    'Active_View_Percent_Visible_At_First_Quartile':
        'FLOAT',
    'Companion_Creative':
        'STRING',
    'Cookie_Reach_Exclusive_Total_Reach':
        'INTEGER',
    'Audio_Mutes':
        'INTEGER',
    'Placement_Rate':
        'STRING',
    'Companion_Clicks':
        'INTEGER',
    'Cookie_Reach_Overlap_Click_Reach':
        'INTEGER',
    'Site_Keyname':
        'STRING',
    'Placement_Cost_Structure':
        'STRING',
    'Percentage_Of_Measurable_Impressions_For_Audio':
        'FLOAT',
    'Rich_Media_Custom_Event_Count':
        'INTEGER',
    'Dbm_Cost_Usd':
        'FLOAT',
    'Dynamic_Element_1_Field_3_Value':
        'STRING',
    'Has_Video_Third_Quartile_Completions':
        'BOOLEAN',
    'Paid_Search_Landing_Page_Url':
        'STRING',
    'Verifiable_Impressions':
        'INTEGER',
    'Average_Time':
        'FLOAT',
    'Creative_Field_12':
        'STRING',
    'Creative_Field_11':
        'STRING',
    'Creative_Field_10':
        'STRING',
    'Channel_Mix':
        'STRING',
    'Paid_Search_Campaign':
        'STRING',
    'Natural_Search_Landing_Page':
        'STRING',
    'Dynamic_Element_1_Field_4_Value':
        'STRING',
    'Payment_Source':
        'STRING',
    'Planned_Media_Cost':
        'FLOAT',
    'Conversion_Referrer':
        'STRING',
    'Companion_Creative_Id':
        'INTEGER',
    'Dynamic_Element_4_Field_2_Value':
        'STRING',
    'Total_Conversions':
        'FLOAT',
    'Custom_Variable_Count_2':
        'INTEGER',
    'Paid_Search_External_Ad_Group_Id':
        'INTEGER',
    'Hd_Video_Player_Size_Impressions':
        'INTEGER',
    'Click_Through_Transaction_Count':
        'FLOAT',
    'Cookie_Reach_Exclusive_Click_Reach_Percent':
        'FLOAT',
    'Floodlight_Attributed_Interaction':
        'STRING',
    'Dynamic_Profile':
        'STRING',
    'Floodlight_Variable_28':
        'STRING',
    'Floodlight_Variable_29':
        'STRING',
    'Dynamic_Element_2_Field_2_Value':
        'STRING',
    'Floodlight_Variable_22':
        'STRING',
    'Floodlight_Variable_23':
        'STRING',
    'Floodlight_Variable_20':
        'STRING',
    'Floodlight_Variable_21':
        'STRING',
    'Floodlight_Variable_26':
        'STRING',
    'Floodlight_Variable_27':
        'STRING',
    'Floodlight_Variable_24':
        'STRING',
    'Floodlight_Variable_25':
        'STRING',
    'Dynamic_Element_4_Field_5_Value':
        'STRING',
    'Creative_Id':
        'INTEGER',
    'Activity_Per_Click':
        'FLOAT',
    'Floodlight_Variable_88':
        'STRING',
    'Active_View_Percent_Of_First_Quartile_Impressions_Visible':
        'FLOAT',
    'Serving_Problems':
        'INTEGER',
    'Placement':
        'STRING',
    'Dynamic_Element_2_Field_Value_1':
        'STRING',
    'Dynamic_Element_2_Field_Value_2':
        'STRING',
    'Dynamic_Element_2_Field_Value_3':
        'STRING',
    'Interaction_Count_Mobile_Video':
        'INTEGER',
    'Has_Full_Screen_Video_Completions':
        'BOOLEAN',
    'Placement_Total_Planned_Media_Cost':
        'STRING',
    'Video_First_Quartile_Completions':
        'INTEGER',
    'Twitter_Creative_Media_Id':
        'INTEGER',
    'Cookie_Reach_Duplicate_Total_Reach':
        'INTEGER',
    'Rich_Media_Impressions':
        'INTEGER',
    'Video_Completions':
        'INTEGER',
    'Month':
        'STRING',
    'Paid_Search_Keyword_Id':
        'INTEGER',
    'Cookie_Reach_Duplicate_Click_Reach_Percent':
        'FLOAT',
    'Replies':
        'INTEGER',
    'Dynamic_Element_5_Field_6_Value':
        'STRING',
    'Video_Mutes':
        'INTEGER',
    'Flight_Booked_Units':
        'STRING',
    'Dynamic_Element_Value_Id':
        'STRING',
    'Expansion_Time':
        'INTEGER',
    'Invalid_Clicks':
        'INTEGER',
    'Has_Video_Progress_Events':
        'BOOLEAN',
    'Dynamic_Element_2_Field_3_Value':
        'STRING',
    'Rich_Media_Standard_Event_Path_Summary':
        'STRING',
    'Video_Muted_At_Start':
        'INTEGER',
    'Audio_Companion_Clicks':
        'INTEGER',
    'Active_View_Audible_Amp_Fully_On_Screen_For_Half_Of_Duration_15_Sec_Cap_Measurable_Impressions':
        'INTEGER',
    'Interaction_Date_Time':
        'DATETIME',
    'User_List':
        'STRING',
    'Event_Timers':
        'FLOAT',
    'Paid_Search_External_Keyword_Id':
        'INTEGER',
    'Timers':
        'INTEGER',
    'Floodlight_Paid_Search_Action_Conversion_Percentage':
        'FLOAT',
    'View_Through_Revenue_Cross_Environment':
        'FLOAT',
    'Advertiser':
        'STRING',
    'Has_Video_Unmutes':
        'BOOLEAN',
    'Natural_Search_Query':
        'STRING',
    'Audio_Plays':
        'INTEGER',
    'Unique_Reach_Average_Impression_Frequency':
        'FLOAT',
    'Path_Type':
        'STRING',
    'Dynamic_Field_Value_2':
        'STRING',
    'Interaction_Channel':
        'STRING',
    'Blocked_Impressions':
        'INTEGER',
    'Dynamic_Field_Value_5':
        'STRING',
    'Dynamic_Field_Value_6':
        'STRING',
    'Placement_Compatibility':
        'STRING',
    'City':
        'STRING',
    'Dbm_Line_Item_Id':
        'INTEGER',
    'Cookie_Reach_Incremental_Click_Reach':
        'INTEGER',
    'Floodlight_Variable_61':
        'STRING',
    'Natural_Search_Processed_Landing_Page_Query_String':
        'STRING',
    'Report_Day':
        'DATE',
    'Dbm_Site':
        'STRING',
    'Connection_Type':
        'STRING',
    'Video_Average_View_Time':
        'FLOAT',
    'Click_Through_Revenue_Cross_Environment':
        'FLOAT',
    'Dbm_Creative':
        'STRING',
    'Attributed_Event_Environment':
        'STRING',
    'Floodlight_Variable_99':
        'STRING',
    'Floodlight_Variable_98':
        'STRING',
    'Measurable_Impressions_For_Video_Player_Size':
        'INTEGER',
    'Dynamic_Element_1_Value_Id':
        'STRING',
    'Paid_Search_Engine_Account_Id':
        'INTEGER',
    'Floodlight_Variable_93':
        'STRING',
    'Floodlight_Variable_92':
        'STRING',
    'Floodlight_Variable_91':
        'STRING',
    'Floodlight_Variable_90':
        'STRING',
    'Floodlight_Variable_97':
        'STRING',
    'Placement_Strategy':
        'STRING',
    'Floodlight_Variable_95':
        'STRING',
    'Floodlight_Variable_94':
        'STRING',
    'Active_View_Percent_Of_Completed_Impressions_Audible_And_Visible':
        'FLOAT',
    'Floodlight_Variable_75':
        'STRING',
    'Floodlight_Variable_74':
        'STRING',
    'Floodlight_Variable_77':
        'STRING',
    'Floodlight_Variable_76':
        'STRING',
    'Floodlight_Variable_71':
        'STRING',
    'Floodlight_Variable_70':
        'STRING',
    'Floodlight_Variable_73':
        'STRING',
    'Activity':
        'STRING',
    'Natural_Search_Engine_Url':
        'STRING',
    'Total_Display_Time':
        'INTEGER',
    'User_List_Description':
        'STRING',
    'Active_View_Impressions_Audible_And_Visible_At_Completion':
        'INTEGER',
    'Floodlight_Variable_79':
        'STRING',
    'Floodlight_Variable_78':
        'STRING',
    'Twitter_Impression_Type':
        'STRING',
    'Active_View_Average_Viewable_Time_Seconds':
        'FLOAT',
    'Active_View_Percent_Measurable_Impressions':
        'FLOAT',
    'Natural_Search_Landing_Page_Query_String':
        'STRING',
    'Percentage_Of_Measurable_Impressions_For_Video_Player_Size':
        'FLOAT',
    'Has_Full_Screen_Impressions':
        'BOOLEAN',
    'Conversion_Id':
        'INTEGER',
    'Creative_Version':
        'STRING',
    'Active_View_Percent_In_Background':
        'FLOAT',
    'Dynamic_Element_2_Value_Id':
        'STRING',
    'Hours_Since_Attributed_Interaction':
        'INTEGER',
    'Dynamic_Element_3_Field_5_Value':
        'STRING',
    'Has_Video_First_Quartile_Completions':
        'BOOLEAN',
    'Dynamic_Element':
        'STRING',
    'Active_View_Percent_Visible_10_Seconds':
        'FLOAT',
    'Booked_Clicks':
        'FLOAT',
    'Booked_Impressions':
        'FLOAT',
    'Tran_Value':
        'STRING',
    'Dynamic_Element_Clicks':
        'INTEGER',
    'Has_Dynamic_Impressions':
        'BOOLEAN',
    'Site_Id_Site_Directory':
        'INTEGER',
    'Active_View_Percent_Audible_And_Visible_At_Third_Quartile':
        'FLOAT',
    'Twitter_Url_Clicks':
        'INTEGER',
    'Dynamic_Element_2_Field_6_Value':
        'STRING',
    'Has_Backup_Image':
        'BOOLEAN',
    'Dynamic_Element_2_Field_5_Value':
        'STRING',
    'Rich_Media_Custom_Event_Path_Summary':
        'STRING',
    'Advertiser_Group':
        'STRING',
    'General_Invalid_Traffic_Givt_Clicks':
        'INTEGER',
    'Has_Video_Pauses':
        'BOOLEAN',
    'Follows':
        'INTEGER',
    'Has_Html5_Impressions':
        'BOOLEAN',
    'Percent_Invalid_Clicks':
        'FLOAT',
    'Percent_Invalid_Impressions':
        'FLOAT',
    'Activity_Date_Time':
        'DATETIME',
    'Site_Site_Directory':
        'STRING',
    'Placement_Pixel_Size':
        'STRING',
    'Within_Floodlight_Lookback_Window':
        'STRING',
    'Dbm_Partner':
        'STRING',
    'Dynamic_Element_3_Field_3_Value':
        'STRING',
    'Ord_Value':
        'STRING',
    'Floodlight_Configuration':
        'INTEGER',
    'Ad_Id':
        'INTEGER',
    'Video_Plays':
        'INTEGER',
    'Video_Player_Location_Avg_Pixels_From_Left':
        'INTEGER',
    'Days_Since_First_Interaction':
        'INTEGER',
    'Event_Counters':
        'INTEGER',
    'Active_View_Not_Measurable_Impressions':
        'INTEGER',
    'Landing_Page_Url':
        'STRING',
    'Ad_Status':
        'STRING',
    'Unique_Reach_Impression_Reach':
        'INTEGER',
    'Dynamic_Element_2_Field_4_Value':
        'STRING',
    'Dbm_Partner_Id':
        'INTEGER',
    'Asset_Id':
        'INTEGER',
    'Video_View_Rate':
        'FLOAT',
    'Twitter_App_Install_Clicks':
        'INTEGER',
    'Cookie_Reach_Duplicate_Total_Reach_Percent':
        'FLOAT',
    'Total_Social_Engagements':
        'INTEGER',
    'Media_Cost':
        'FLOAT',
    'Placement_Tag_Type':
        'STRING',
    'Dbm_Insertion_Order':
        'STRING',
    'Floodlight_Variable_39':
        'STRING',
    'Floodlight_Variable_38':
        'STRING',
    'Paid_Search_External_Ad_Id':
        'INTEGER',
    'Browser_Platform':
        'STRING',
    'Floodlight_Variable_31':
        'STRING',
    'Floodlight_Variable_30':
        'STRING',
    'Floodlight_Variable_33':
        'STRING',
    'Floodlight_Variable_32':
        'STRING',
    'Floodlight_Variable_35':
        'STRING',
    'Floodlight_Variable_34':
        'STRING',
    'Floodlight_Variable_37':
        'STRING',
    'Floodlight_Variable_36':
        'STRING',
    'Dynamic_Element_View_Through_Conversions':
        'INTEGER',
    'Active_View_Viewable_Impression_Cookie_Reach':
        'INTEGER',
    'Video_Interaction_Rate':
        'FLOAT',
    'Dynamic_Element_3_Field_1_Value':
        'STRING',
    'Booked_Activities':
        'FLOAT',
    'Has_Video_Full_Screen':
        'BOOLEAN',
    'User_List_Membership_Life_Span':
        'STRING',
    'Video_Length':
        'STRING',
    'Paid_Search_Keyword':
        'STRING',
    'Revenue_Per_Click':
        'FLOAT',
    'Downloaded_Impressions':
        'INTEGER',
    'Days_Since_Attributed_Interaction':
        'INTEGER',
    'Code_Serves':
        'INTEGER',
    'Effective_Cpm':
        'FLOAT',
    'Environment':
        'STRING',
    'Active_View_Percent_Play_Time_Audible_And_Visible':
        'FLOAT',
    'Paid_Search_Agency':
        'STRING',
    'Dynamic_Element_5_Field_3_Value':
        'STRING',
    'Paid_Search_Engine_Account_Category':
        'STRING',
    'Week':
        'STRING',
    'Designated_Market_Area_Dma':
        'STRING',
    'Cookie_Reach_Click_Reach':
        'INTEGER',
    'Twitter_Buy_Now_Purchases':
        'INTEGER',
    'Floodlight_Paid_Search_Average_Dcm_Transaction_Amount':
        'FLOAT',
    'Placement_Start_Date':
        'DATE',
    'Cookie_Reach_Overlap_Impression_Reach_Percent':
        'FLOAT',
    'View_Through_Revenue':
        'FLOAT',
    'Impression_Delivery_Status':
        'FLOAT',
    'Floodlight_Paid_Search_Transaction_Conversion_Percentage':
        'FLOAT',
    'Click_Through_Conversions_Cross_Environment':
        'INTEGER',
    'Activity_Id':
        'INTEGER',
    'Has_Video_Mutes':
        'BOOLEAN',
    'Exits':
        'INTEGER',
    'Paid_Search_Bid_Strategy':
        'STRING',
    'Active_View_Percent_Of_Midpoint_Impressions_Visible':
        'FLOAT',
    'Dbm_Insertion_Order_Id':
        'INTEGER',
    'Placement_Id':
        'INTEGER',
    'App_Id':
        'STRING',
    'View_Through_Transaction_Count':
        'FLOAT',
    'Floodlight_Paid_Search_Transaction_Revenue_Per_Spend':
        'FLOAT',
    'View_Through_Conversions':
        'FLOAT',
    'Dynamic_Element_5_Field_4_Value':
        'STRING',
    'Active_View_Percent_Of_First_Quartile_Impressions_Audible_And_Visible':
        'FLOAT',
    'Companion_Creative_Pixel_Size':
        'STRING',
    'Revenue_Per_Thousand_Impressions':
        'FLOAT',
    'Natural_Search_Clicks':
        'INTEGER',
    'Dynamic_Element_3_Field_2_Value':
        'STRING',
    'Audio_First_Quartile_Completions':
        'INTEGER',
    'Dynamic_Element_1_Field_Value_3':
        'STRING',
    'Dynamic_Element_1_Field_Value_2':
        'STRING',
    'Dynamic_Element_1_Field_Value_1':
        'STRING',
    'Full_Screen_Average_View_Time':
        'FLOAT',
    'Dynamic_Element_5_Value_Id':
        'STRING',
    'Rich_Media_Click_Rate':
        'FLOAT',
    'User_List_Id':
        'INTEGER',
    'Click_Through_Url':
        'STRING',
    'Active_View_Percent_Of_Completed_Impressions_Visible':
        'FLOAT',
    'Video_Prominence_Score':
        'STRING',
    'Active_View_Percent_Play_Time_Audible':
        'FLOAT',
    'Natural_Search_Actions':
        'FLOAT',
    'Platform_Type':
        'STRING',
    'General_Invalid_Traffic_Givt_Impressions':
        'INTEGER',
    'Active_View_Percent_Audible_And_Visible_At_First_Quartile':
        'FLOAT',
    'Dynamic_Element_3':
        'STRING',
    'Active_View_Percent_Play_Time_Visible':
        'FLOAT',
    'Active_View_Percent_Of_Third_Quartile_Impressions_Audible_And_Visible':
        'FLOAT',
    'Paid_Search_Transactions':
        'FLOAT',
    'Rich_Media_Event':
        'STRING',
    'Country':
        'STRING',
    'Dynamic_Element_3_Field_6_Value':
        'STRING',
    'Expansions':
        'INTEGER',
    'Interaction_Rate':
        'FLOAT',
    'Natural_Search_Processed_Landing_Page':
        'STRING',
    'Floodlight_Impressions':
        'INTEGER',
    'Paid_Search_Bid_Strategy_Id':
        'INTEGER',
    'Interactive_Impressions':
        'INTEGER',
    'Interaction_Count_Natural_Search':
        'INTEGER',
    'Twitter_Impression_Id':
        'INTEGER',
    'Ad':
        'STRING',
    'Paid_Search_Ad_Group_Id':
        'INTEGER',
    'Paid_Search_Campaign_Id':
        'INTEGER',
    'Full_Screen_Video_Completions':
        'INTEGER',
    'Dynamic_Element_Value':
        'STRING',
    'State_Region':
        'STRING',
    'Placement_End_Date':
        'DATE',
    'Cookie_Reach_Duplicate_Impression_Reach_Percent':
        'FLOAT',
    'Paid_Search_Impressions':
        'INTEGER',
    'Cookie_Reach_Average_Impression_Frequency':
        'FLOAT',
    'Natural_Search_Engine_Country':
        'STRING',
    'Paid_Search_Revenue':
        'FLOAT',
    'Percent_Composition_Impressions':
        'FLOAT',
}
