# Generated by Django 2.2.13 on 2020-12-02 15:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('project', '0002_project_key'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='service',
            field=models.TextField(default=''),
        ),
    ]
