###########################################################################
#
#  Copyright 2020 Google LLC
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
###########################################################################

Placement_Cost_Schema = [{
    'name': 'Advertiser_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Placement_Comments',
    'type': 'STRING',
    'mode': 'NULLABLE'
}, {
    'name': 'Placement_End_Date',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Placement_Rate',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Placement_Start_Date',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Placement_Total_Booked_Units',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Placement_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Floodlight_Config_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}, {
    'name': 'Account_Id',
    'type': 'INTEGER',
    'mode': 'NULLABLE'
}]
